﻿Option Strict On
Imports CSCore
Imports CSCore.SoundOut
Imports CSCore.Codecs
Imports CSCore.Streams
Imports System.Windows.Forms

Public Class Form1

    Private soundOut As ISoundOut
    Private soundSource As IWaveSource
    Private isPositionScrolling As Boolean = False

    Private Sub PlayButton_Click(sender As Object, e As EventArgs) Handles playButton.Click
        If soundOut IsNot Nothing AndAlso soundOut.PlaybackState = PlaybackState.Playing Then
            soundOut.Pause()
        ElseIf soundOut IsNot Nothing AndAlso soundOut.PlaybackState = PlaybackState.Paused Then
            soundOut.Resume()
        Else 'stopped
            OpenFile()
        End If
    End Sub

    Private Sub StopButton_Click(sender As Object, e As EventArgs) Handles stopButton.Click
        StopPlayback()
    End Sub

    Private Sub Form1_FormClosing(sender As Object, e As Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        StopPlayback()
    End Sub

    Private Sub OpenFile()
        Dim openfiledialog As New OpenFileDialog()
        openfiledialog.Filter = CodecFactory.SupportedFilesFilterEN

        If openfiledialog.ShowDialog() = Windows.Forms.DialogResult.OK Then
            StopPlayback()

            Dim source As IWaveSource = Nothing
            Try
                source = CodecFactory.Instance.GetCodec(openfiledialog.FileName)
            Catch ex As Exception
                MessageBox.Show("Dateityp wird nicht unterstützt.")
                Return
            End Try

            Dim sampleSource As ISampleSource = New NotificationSource(source)
            CType(sampleSource, NotificationSource).Interval = 100 '100 ms interval
            AddHandler CType(sampleSource, NotificationSource).BlockRead, AddressOf OnBlockRead

            Dim source1 = sampleSource.ToWaveSource(16)
            source1 = New LoopStream(source1)
            soundSource = source1

            soundOut = New DirectSoundOut()
            soundOut.Initialize(source1)
            soundOut.Play()
        End If
    End Sub

    Private Sub StopPlayback()
        If soundOut IsNot Nothing Then
            soundOut.Stop()
            soundOut.Dispose()
            soundOut = Nothing
        End If
        If soundSource IsNot Nothing Then
            soundSource.Dispose()
            soundSource = Nothing
        End If
    End Sub

Private Sub OnBlockRead(ByVal sender As Object, ByVal e As BlockReadEventArgs(Of Single))
    Dim source As IWaveSource = soundSource
    Dim text = String.Format("{0:mm\:ss} \ {1:mm\:ss}", source.GetPosition(), source.GetLength())

    Dim v As Double = CType(source.Position, Double) / CType(source.Length, Double)
    Me.Invoke(Sub()
        Label1.Text = text
        If Not isPositionScrolling Then
            positionTrackbar.Value = CType(v * positionTrackbar.Maximum, Integer)
        End If
    End Sub)
End Sub

    Private Sub volumenTrackbar_Scroll(sender As Object, e As EventArgs) Handles volumenTrackbar.Scroll
        If soundOut IsNot Nothing AndAlso soundOut.PlaybackState <> PlaybackState.Stopped Then
            soundOut.Volume = CSng(volumenTrackbar.Value / 100.0)
        End If
    End Sub

    Private Sub positionTrackbar_Scroll(sender As Object, e As EventArgs) Handles positionTrackbar.Scroll
        If soundSource IsNot Nothing Then
            Dim v As Double = CType(positionTrackbar.Value, Double) / CType(positionTrackbar.Maximum, Double)
            Dim position As Long = CType(soundSource.Length * v, Long)
            soundSource.Position = position
        End If
    End Sub

    Private Sub positionTrackbar_MouseDown(sender As Object, e As MouseEventArgs) Handles positionTrackbar.MouseDown
        isPositionScrolling = True
    End Sub

    Private Sub positionTrackbar_MouseUp(sender As Object, e As MouseEventArgs) Handles positionTrackbar.MouseUp
        isPositionScrolling = False
    End Sub
End Class